@@ -4,8 +4,8 @@ use v5.26;
 
 package ChordPro;
 
-use ChordPro::Logger;
-use ChordPro::Files;
+use App::Packager;
+
 use ChordPro::Utils;
 use ChordPro::Chords;
 use ChordPro::Output::Common;
@@ -39,16 +39,16 @@ Typical ChordPro input:
 
     {start_of_chorus}
     Swing [D]low, sweet [G]chari[D]ot,
-    Comin' for to carry me [A7]home.
+    Comin’ for to carry me [A7]home.
     Swing [D7]low, sweet [G]chari[D]ot,
-    Comin' for to [A7]carry me [D]home.
+    Comin’ for to [A7]carry me [D]home.
     {end_of_chorus}
 
     # Verse
     I [D]looked over Jordan, and [G]what did I [D]see,
-    Comin' for to carry me [A7]home.
-    A [D]band of angels [G]comin' after [D]me,
-    Comin' for to [A7]carry me [D]home.
+    Comin’ for to carry me [A7]home.
+    A [D]band of angels [G]comin’ after [D]me,
+    Comin’ for to [A7]carry me [D]home.
 
     {c: Chorus}
 
@@ -75,26 +75,9 @@ our $config;
 
 package ChordPro;
 
-use ChordPro::Paths;
-use Encode qw(decode_utf8);
-
-sub xximport {
-    # Add private library.
-    my $lib = CP->privlib;
-    for ( @INC ) {
-	return if $_ eq $lib;
-    }
-    unshift( @INC, $lib );
-}
-
 sub ::run {
     binmode(STDERR, ':utf8');
     binmode(STDOUT, ':utf8');
-    for ( @ARGV ) {
-	next if ref ne "";
-	$_ = decode_utf8($_);
-    }
-
     $options = app_setup( "ChordPro", $VERSION );
     $options->{trace}   = 1 if $options->{debug};
     $options->{verbose} = 1 if $options->{trace};
@@ -115,16 +98,6 @@ sub chordpro {
     # Establish backend.
     my $of = $options->{output};
 
-    # Progress callback, if any.
-    progress( callback => $options->{progress_callback}, index => -1 )
-      if $options->{progress_callback};
-
-    if ( $options->{'convert-config'} ) {
-	die("\"--convert-config\" requires a single config file name\n")
-	  if @ARGV;
-	return ChordPro::Config::convert_config( $options->{'convert-config'}, $of );
-    }
-
     if ( defined($of) && $of ne "" ) {
         if ( $of =~ /\.pdf$/i ) {
             $options->{generate} ||= "PDF";
@@ -154,9 +127,6 @@ sub chordpro {
         elsif ( $of =~ /\.(md|markdown)$/i ) {
             $options->{generate} ||= "Markdown";
         }
-        elsif ( $of =~ /\.meta$/i ) {
-            $options->{generate} ||= "Meta";
-        }
         elsif ( $of =~ /\.(debug)$/i ) {
             $options->{generate} ||= "Debug";
         }
@@ -168,7 +138,9 @@ sub chordpro {
 	    # No default if more than one input document.
 	    die("Please use \"--output\" to specify the output file name\n");
 	}
-	my $f = CP->sibling( $ARGV[0], ext => ".pdf" );
+	my $f = $ARGV[0];
+	$f =~ s/\.\w+$/.pdf/;
+	$f .= ".pdf" if $f eq $ARGV[0];
 	$options->{output} = $f;
 	warn("Writing output to $f\n") if $options->{verbose};
     }
@@ -203,58 +175,27 @@ sub chordpro {
 
     # Check for metadata in filelist. Actually, this works on the
     # command line as well, but don't tell anybody.
-    progress( phase => "Parsing", index => 0,
-	      total => 0+grep { !/^--/ } @ARGV )
-      if @ARGV > 1;
-
-    my %gopts;
     foreach my $file ( @ARGV ) {
-
-	my @w = ( $file );
-	if ( $file =~ /(^|\s)--\w+/ || $file =~ /^["']/ ) {
-	    @w = Text::ParseWords::shellwords($file);
+	my $opts;
+	if ( $file =~ /(^|\s)--(?:meta|config|define)\b/ ) {
+	    # Break into words.
+	    my @w = Text::ParseWords::shellwords($file);
+	    my %meta;
+	    my %defs;
+	    my @cfg;
+	    die("Error in filelist: $file\n")
+	      unless Getopt::Long::GetOptionsFromArray
+	      ( \@w, 'config=s@' => \@cfg, 'meta=s%' => \%meta,
+		'define=s%' => \%defs,
+	      )
+	      && @w == 1;
+	    $file = $w[0];
+	    $opts = { meta => { map { $_, [ $meta{$_} ] } keys %meta },
+		      defs => \%defs };
+	    if ( @cfg ) {
+		$opts->{meta}->{__config} = \@cfg;
+	    }
 	}
-	my %meta;
-	my %defs;
-	my @cfg;
-	my %opts;
-	die("Error in filelist: $file\n")
-	  unless Getopt::Long::GetOptionsFromArray
-	  ( \@w, \%opts, 'config=s@' => \@cfg, 'meta=s%' => \%meta,
-	    'define=s%' => \%defs,
-	    'title=s', 'subtitle=s', 'dir:s', 'filelist:s',
-	  )
-	  && (    ( @w == 1 && ! keys(%opts) ) # filename
-	       || ( @w == 0 &&   keys(%opts) ) # options
-	     );
-
-	for ( qw( title subtitle ) ) {
-	    next unless defined $opts{$_};
-	    $options->{$_} = $opts{$_};
-	}
-	for ( qw( filelist dir ) ) {
-	    next unless defined $opts{$_};
-	    $gopts{$_} = $opts{$_} eq "" ? undef : $opts{$_};
-	}
-	unless ( @w ) {
-	    progress( msg => $file ) if @ARGV > 1 && $file !~ /^--/;
-	    next;
-	}
-
-	$file = $w[0];
-	if ( defined($gopts{dir})
-	     && !fn_is_absolute($file) ) {
-	    $file = fn_catfile( $gopts{dir}, $file );
-	}
-	my $opts = { meta => { map { $_, [ $meta{$_} ] } keys %meta },
-		     defs => \%defs };
-	if ( @cfg ) {
-	    $opts->{meta}->{__config} = \@cfg;
-	}
-	$opts->{generate} = $options->{generate};
-	# Wx runs on temp files, so pass real filename in.
-	$opts->{filesource} = $options->{filesource};
-	progress( msg => $file ) if @ARGV > 1;
 	$s->parse_file( $file, $opts );
     }
 
@@ -317,7 +258,8 @@ sub chordpro {
     # array of lines to be written.
     if ( $res && @$res > 0 ) {
         if ( $of && $of ne "-" ) {
-            my $fd = fs_open( $of, '>:utf8' );
+            open( my $fd, '>', $of );
+	    binmode( $fd, ":utf8" );
 	    push( @$res, '' ) unless $res->[-1] eq '';
 	    print { $fd } ( join( "\n", @$res ) );
 	    close($fd);
@@ -332,8 +274,19 @@ sub chordpro {
 }
 
 sub ::dump {
-    use ChordPro::Dumper;
-    ddp(@_);
+    use Data::Dumper qw();
+    local $Data::Dumper::Sortkeys  = 1;
+    local $Data::Dumper::Indent    = 1;
+    local $Data::Dumper::Quotekeys = 0;
+    local $Data::Dumper::Deparse   = 1;
+    local $Data::Dumper::Terse     = 1;
+    local $Data::Dumper::Trailingcomma = 1;
+    local $Data::Dumper::Useperl = 1;
+    local $Data::Dumper::Useqq     = 0; # I want unicode visible
+
+    my $s = Data::Dumper::Dumper @_;
+    defined wantarray or warn $s;
+    return $s;
 }
 
 ################ Options and Configuration ################
@@ -363,10 +316,7 @@ active.
 
 =item B<--cover=>I<FILE>
 
-Prepends the contents of the named PDF document to the output. This can
-be used to add cover pages.
-
-See also B<--title>.
+See B<--front-matter>.
 
 =item B<--csv>
 
@@ -403,10 +353,10 @@ This option may be specified multiple times.
 Song file names listed on the command line are processed I<after> the
 files from the filelist arguments.
 
-=item B<--front-matter=>I<FILE>
+=item B<--front-matter=>I<FILE> B<--cover=>I<FILE>
 
 Prepends the contents of the named PDF document to the output. This can
-be used to produce documents with front matter pages.
+be used to produce documents with front matter (cover) pages.
 
 =item B<--lyrics-only> (short: B<-l>)
 
@@ -466,19 +416,6 @@ Requires the input to be strictly compliant to the ChordPro standard.
 
 This is enabled by default. See also B<--nostrict>.
 
-=item B<--subtitle=>I<XXX>
-
-Subtitle (for songbooks).
-
-=item B<--title=>I<XXX>
-
-Title (for songbooks).
-
-If specified and a table of contents is requested, a nice coverpage
-will be added.
-
-Note that B<--title> overrides B<--cover>.
-
 =item B<--toc> (short: B<-i>)
 
 Includes a table of contents.
@@ -569,11 +506,11 @@ Prints chord diagrams of all user defined chords used in a song.
 
 =item B<--even-pages-number-left> (short B<-L>)
 
-Not supported. Use C<pdf.even-odd-pages> in the config instead.
+Prints even/odd pages with pages numbers left on even pages.
 
 =item B<--odd-pages-number-left>
 
-Not supported. Use C<pdf.even-odd-pages> in the config instead.
+Prints even/odd pages with pages numbers left on odd pages.
 
 =item B<--page-size=>I<FMT> (short: B<-P>) *
 
@@ -677,7 +614,7 @@ Don't use the specific config file, even if it exists.
 Sets a configuration item. I<item> must be in the format of
 colon-separated configuration keys, an equal sign, and the value. For
 example, the equivalent of B<--diagrams=none> is
-B<--define=diagrams.show=0>.
+B<--define=diagrams:show=0>.
 
 B<--define> may be used multiple times to set multiple items.
 
@@ -689,39 +626,18 @@ the command line.
 This guarantees that the program is running with the default
 configuration.
 
-=item B<--print-template-config>
-
-Prints a simplified template configuration to standard output, and
-exits. The configuration is commented to explain its contents.
-
-The config contains most of the ChordPro configuration items, all
-commented out. It is easy to get started with configuring ChordPro
-by enabling and modifyng just a few items at a time.
-
 =item B<--print-default-config>
 
-If used once, behaves like `--print-template-config`. This is to avoid
-confusing novice users.
+Prints the default configuration to standard output, and exits.
 
-To get the full default configuration, repeat this option.
+The default configuration is commented to explain its contents.
 
 =item B<--print-final-config>
 
 Prints the final configuration (after processing all system, user and
 other config files)  to standard output, and exits.
 
-=item B<--convert-config=>I<file>
-
-This option requires a single file name argument.
-
-The specified config file is loaded and immedeately written out in
-standard format, including schema comments. After this, the program
-exits. B<No songs are processed.>
-
-Output goes to standard output unless `--output` is used to specify an
-alternative.
-
-All other options are ignored.
+The final configuration is not commented. Sorry.
 
 =back
 
@@ -750,6 +666,8 @@ Provides more verbose information of what is going on.
 =cut
 
 use Getopt::Long 2.13 qw( :config no_ignorecase );
+use File::Spec;
+use File::LoadLines;
 
 # Package name.
 my $my_package;
@@ -767,7 +685,6 @@ sub app_setup {
     my $defcfg = 0;		# handled locally
     my $fincfg = 0;		# handled locally
     my $deltacfg = 0;		# handled locally
-    my $tmplcfg = 0;		# handled locally
     my $dump_chords = 0;	# handled locally
 
     # Package name.
@@ -787,7 +704,37 @@ sub app_setup {
     }
 
     # Config files.
-    %configs = %{ CP->configs };
+    if ( -d "/etc" ) {          # some *ux
+        $configs{sysconfig} =
+          File::Spec->catfile( "/", "etc", "$app_lc.json" );
+    }
+
+    if ( $ENV{XDG_CONFIG_HOME} && -d $ENV{XDG_CONFIG_HOME} ) {
+	$configs{userconfig} =
+	  File::Spec->catfile( $ENV{XDG_CONFIG_HOME}, $app_lc, "$app_lc.json" );
+	$ENV{CHORDPRO_LIB} ||= File::Spec->catfile( $ENV{XDG_CONFIG_HOME}, $app_lc);
+    }
+    elsif ( $ENV{HOME} && -d $ENV{HOME} ) {
+	my $dir = File::Spec->catfile( $ENV{HOME}, ".config" );
+        if ( -d $dir ) {
+            $configs{userconfig} =
+              File::Spec->catfile( $dir, $app_lc, "$app_lc.json" );
+	    $ENV{CHORDPRO_LIB} ||= File::Spec->catfile( $dir, $app_lc );
+        }
+        else {
+	    $dir = File::Spec->catfile( $ENV{HOME}, ".$app_lc" );
+            $configs{userconfig} =
+              File::Spec->catfile( $dir, "$app_lc.json" );
+	    $ENV{CHORDPRO_LIB} ||= $dir;
+        }
+    }
+
+    if ( -s ".$app_lc.json" ) {
+        $configs{config} = ".$app_lc.json";
+    }
+    else {
+        $configs{config} = "$app_lc.json";
+    }
 
     my $options =
       {
@@ -839,24 +786,17 @@ sub app_setup {
 	  "diagrams=s",			# Prints chord diagrams
           "encoding=s",
 	  "csv!",			# Generates contents CSV
-	  "front-matter=s",		# Front matter page(s)
-	  "cover=s",			# Cover page(s)
-	  "back-matter|back=s",		# Back matter page(s)
+	  "front-matter|cover=s",	# Front matter page(s)
+	  "back-matter=s",		# Back matter page(s)
 	  "filelist=s@",		# List of input files
-	  "title=s",			# Title (for books)
-	  "subtitle=s",			# Subtitle (for books)
 	  "meta=s\%",			# Command line meta data
 	  "decapo",			# remove capo
 	  "fragment|F",			# partial (incomplete) song
 	  "strict!",			# strict conformance
 
-	  ### Experimental ###
+          ### Standard Chordii Options ###
 
-	  "progress_callback=s",
-
-	  ### Standard Chordii Options ###
-
-          "about|A+" => \$about,        # About...
+          "about|A" => \$about,         # About...
           "chord-font|C=s",             # Sets chord font
           "chord-grid-size|s=f",        # Sets chord diagram size [30]
           "chord-grids-sorted|S!",      # Prints chord diagrams ordered
@@ -898,15 +838,12 @@ sub app_setup {
           'nouserconfig|no-userconfig',
 	  'nodefaultconfigs|no-default-configs|X',
 	  'define=s%',
-	  'print-default-config+' => \$defcfg,
-	  'print-final-config'    => \$fincfg,
-	  'print-delta-config'    => \$deltacfg,
-	  'print-template-config' => \$tmplcfg,
-	  'convert-config=s',
+	  'print-default-config' => \$defcfg,
+	  'print-final-config'   => \$fincfg,
+	  'print-delta-config'   => \$deltacfg,
 
 	  # This aborts option scanning.
-	  'reference|R'		 => sub { $reference++;
-					  CORE::die("!FINISH!"); },
+	  'reference|R'		 => sub { $reference++; die("!FINISH!"); },
 
           ### Standard options ###
 
@@ -915,7 +852,6 @@ sub app_setup {
           'help-config'         => sub { $manual = 2 },
           'manual'              => \$manual,
           'verbose|v+',
-	  I			=> sub { $clo->{progress_callback} = "warn" },
           'trace',
           'debug+',
 
@@ -954,7 +890,6 @@ sub app_setup {
 
     $clo->{trace} ||= $clo->{debug};
     $clo->{verbose} ||= $clo->{trace};
-    $clo->{progress_callback} //= "warn" if $clo->{verbose};
 
     unless ( $ok ) {
         # GNU convention: message to STDERR upon failure.
@@ -965,8 +900,8 @@ sub app_setup {
         # Load Pod::Usage only if needed.
         require Pod::Usage;
         Pod::Usage->import;
-	my $f = $manual == 2 ? "Config.pod" : "ChordPro.pod";
-        unshift( @_, -input => CP->findres( $f, class => "pod" ) );
+	my $f = $manual == 2 ? "pod/Config.pod" : "pod/ChordPro.pod";
+        unshift( @_, -input => getresource($f) );
         &pod2usage;
     };
 
@@ -977,6 +912,7 @@ sub app_setup {
         $pod2usage->(VERBOSE => 2) if $manual;
     }
     app_ident(\*STDOUT, 0) if $version;
+    app_about(\*STDOUT, 0) if $about;
 
     # If the user specified a config, it must exist.
     # Otherwise, set to a default.
@@ -997,13 +933,11 @@ sub app_setup {
         for ( $clo->{$config} ) {
             if ( defined($_) ) {
                 foreach my $c ( @$_ ) {
-		    my $try = $c;
 		    # Check for resource names.
-		    if ( !fs_test( 'r', $try ) ) {
-			$try = CP->findcfg($c);
+		    if ( $c !~ m;[/.]; ) {
+			$c = ::rsc_or_file( $c, "config" );
 		    }
-                    die("$c: $!\n") unless $try && fs_test( 'r', $try );
-		    $c = $try;
+                    die("$c: $!\n") unless -r $c;
                 }
                 next;
             }
@@ -1011,7 +945,7 @@ sub app_setup {
 	    next if $clo->{nodefaultconfigs};
 	    next unless $configs{$config};
             $_ = [ $configs{$config} ];
-            undef($_) unless fs_test( 'fr', $_->[0] );
+            undef($_) unless -r -f $_->[0];
         }
     }
     # If no config was specified, and no default is available, force no.
@@ -1020,39 +954,29 @@ sub app_setup {
     }
     $clo->{nosongconfig} ||= $clo->{nodefaultconfigs};
 
+    # Decode command line strings.
+    # File names are dealt with elsewhere.
+    for ( qw(transcode) ) {
+	next unless defined $clo->{$_};
+	$clo->{$_} = decode_utf8($clo->{$_});
+    }
+    ####TODO: Should decode all, and remove filename exception.
+    for ( keys %{ $clo->{define} } ) {
+	$clo->{define}->{$_} = decode_utf8($clo->{define}->{$_});
+    }
+
     # Plug in command-line options.
     @{$options}{keys %$clo} = values %$clo;
     $::options = $options;
     # warn(::dump($options), "\n") if $options->{debug};
 
-    # To avoid confusion, produce a template if the user asks for a
-    # default config.
-    # Unless she insists...
-    if ( $defcfg >= 2 ) {
-	use File::Copy;
-	my $cfg = fn_catfile( CP->findresdirs("config")->[-1],
-			      "chordpro.json" );
-	binmode STDOUT => ':raw';
-	copy( $cfg, \*STDOUT );
+    if ( $defcfg || $fincfg || $deltacfg ) {
+	print ChordPro::Config::config_default()
+	  if $defcfg;
+	print ChordPro::Config::config_final($deltacfg)
+	  if $fincfg || $deltacfg;
 	exit 0;
     }
-    if ( $tmplcfg || $defcfg ) {
-	use File::Copy;
-	my $cfg = fn_catfile( CP->findresdirs("config")->[-1],
-			      "config.tmpl" );
-	binmode STDOUT => ':raw';
-	copy( $cfg, \*STDOUT );
-	exit 0;
-    }
-    if ( $fincfg || $deltacfg ) {
-	print ChordPro::Config::config_final( delta   => $deltacfg );
-	exit 0;
-    }
-
-    if ( $about ) {
-	$::config = ChordPro::Config::configurator({});
-	app_about( \*STDOUT, $about, 0 );
-    }
 
     if ( $dump_chords ) {
 	$::config = ChordPro::Config::configurator($options);
@@ -1064,17 +988,13 @@ sub app_setup {
     if ( $clo->{filelist} ) {
 	my @files;
 	foreach ( @{ $clo->{filelist} } ) {
-	    push( @files, "--filelist=" . qquote($_) );
-	    my $dir = fn_dirname($_);
-	    my $list = fs_load( $_, $clo );
-	    push( @files, "--dir=" . qquote($dir) );
+	    my $list = loadlines( $_, $clo );
 	    foreach ( @$list ) {
 		next unless /\S/;
 		next if /^#/;
 		s/[\r\n]+$//;
-		push( @files, $_ );
+		push( @files, encode_utf8($_) );
 	    }
-	    push( @files, "--filelist", "--dir" );
 	}
 	if ( @files ) {
 	    if ( $files[0] =~ /\.pdf$/i ) {
@@ -1092,10 +1012,7 @@ sub app_setup {
     # At this point, there should be filename argument(s)
     # unless we're embedded or just dumping chords.
     app_usage(\*STDERR, 1)
-      unless $::__EMBEDDED__
-      || $clo->{'dump-chords'}
-      || $clo->{'convert-config'}
-      || @ARGV;
+      unless $::__EMBEDDED__ || $clo->{'dump-chords'} || @ARGV;
 
     # Return result.
     $options;
@@ -1103,27 +1020,13 @@ sub app_setup {
 
 sub app_ident {
     my ($fh, $exit) = @_;
-    print {$fh} ("This is ChordPro ",
-		 $::options->{reference} ? "reference " : "core ",
-		 $VERSION,
-		 $VERSION =~ /_/ ? " (Unsupported development snapshot)" : "",
-		 "\n"
-		);
+    print {$fh} ("This is ", ::runtimeinfo("short"), "\n");
     exit $exit if defined $exit;
 }
 
 sub app_about {
-    my ($fh, $level, $exit) = @_;
-
-    if ( $level > 2 ) {
-	require JSON::XS;
-	select $fh; $| = 1;	# otherwise no output on MacOS
-	print ${fh} ( JSON::XS->new->canonical->
-		      # pretty->
-		      utf8->convert_blessed->encode(runtime_info()) );
-    }
-    else {
-	print ${fh} <<EndOfAbout,
+    my ($fh, $exit) = @_;
+    print ${fh} <<EndOfAbout;
 
 ChordPro: A lyrics and chords formatting program.
 
@@ -1138,249 +1041,93 @@ To learn more about ChordPro, look for the man page or do
 For more information, see https://www.chordpro.org .
 
 Run-time information:
+@{[::runtimeinfo()]}
 EndOfAbout
-      ::runtimeinfo( $level > 1 ? "extensive" : "normal" );
-    }
     exit $exit if defined $exit;
 }
 
-use List::Util qw(uniq);
+use Cwd qw(realpath);
 
 sub ::runtimeinfo {
-    my $level = shift // "normal";
-    my %i = %{runtime_info($level)};
-    my $fmt0   = "  %-26.26s %-10s";
-    my $fmt2   = $fmt0 . "\n";
-    my $fmt3   = $fmt0 . " (%s)\n";
+    my $short = shift;
 
-    my $msg;
-    for ( $i{general}->{chordpro} ) {
-	if ( $_->{aux} ) {
-	    $msg = sprintf( $fmt3,
-			    "ChordPro " . ($_->{type}//"core"), $_->{version},
-			    $_->{aux} );
-	}
-	else {
-	    $msg = sprintf( $fmt2,
-			    "ChordPro " . ($_->{type}//"core"), $_->{version} );
-	}
-    }
-    if ( $level eq "short" ) {	# used for PDF Creator info
-	$msg =~ s/^\s+//;
-	$msg =~ s/\s+/ /g;
-	$msg =~ s/\n+$//;
-	return $msg;
-    }
-
-    for ( $i{general}{perl} ) {
-	$msg .= sprintf( $fmt3, "Perl", $_->{version}, $_->{dppath} );
-    }
-
-    for ( $i{general}{packager} ) {
-	next unless defined;
-	$msg .= sprintf( $fmt2, $_->{packager}." Packager", $_->{version} );
-    }
-
-    # Determine resource path.
-    my @p;
-    my $tag = "CHORDPRO_LIB";
-    for ( @{$i{general}{library}} ) {
-	$msg .= sprintf( $fmt2, $tag, $_->{dppath} );
-	$tag = "";
-    }
-    for ( $i{general}{xdg_home} ) {
-	next unless defined;
-	$msg .= sprintf( $fmt2, "XDG_CONFIG_HOME", $_->{dppath} );
-    }
-
-    $tag = "Resource path";
-    for ( @{$i{resources}} ) {
-	$msg .= sprintf( $fmt2, $tag, $_->{dppath} );
-	$tag = "";
-    }
-
-    $tag = "FONTDIR";
-    for ( @{$i{general}{fontdir}//[]} ) {
-	next unless defined;
-	$msg .= sprintf( $fmt2, $tag, $_->{dppath} );
-	$tag = "";
-    }
-
-    for ( $i{general}{abc} ) {
-	next unless defined;
-	$msg .= sprintf( $fmt2, "ABC support", $_ );
-    }
-
-    $msg .= "\nModules and libraries:\n";
-    for ( @{$i{modules}} ) {
-	if ( $level eq "normal" ) {
-	    $msg .= sprintf( $fmt2, $_->{name}, $_->{version} );
-	}
-	else {
-	    if ( defined $_->{dppath} ) {
-		$msg .= sprintf( $fmt3, $_->{name}, $_->{version}, $_->{dppath} );
-	    }
-	    else {
-		$msg .= sprintf( $fmt2, $_->{name}, $_->{version} );
-	    }
-	}
-	if ( $_->{library} ) {
-	    if ( $_->{library} =~ /i$/ ) {
-		$msg .= sprintf( $fmt3, "  library", $_->{library}, "embedded" );
-	    }
-	    else {
-		$msg .= sprintf( $fmt2, "  library", $_->{library} );
-	    }
-	}
-    }
-
-    return $msg;
-}
-
-# Gather runtime details.
-sub runtime_info {
-    my $level = shift // "normal";
-    my $cp = ChordPro::Paths->get;
-    my $res;
+    my $fmt   = "  %-22.22s %s\n";
+    my $fmtv  = defined($Wx::VERSION) ? "  %s version %s\n" : $fmt;
+    my $fmtvv = defined($Wx::VERSION) ? "  %s %s\n" : $fmt;
 
     # Sometimes version numbers are localized...
     my $dd = sub { my $v = $_[0]; $v =~ s/,/./g; $v };
 
-    for ( $res->{general}{chordpro} ) {
-	$_->{version} = $dd->($VERSION);
-	$_->{type} = $::options->{reference} ? "reference" : "core";
-	$_->{aux} = "Unsupported development snapshot"
-	  if $VERSION =~ /_/;
+    my $msg = sprintf( $fmtv, "ChordPro core", $dd->($VERSION) );
+    $msg =~ s/core/reference/ if $::options->{reference};
+    if ( $VERSION =~ /_/ ) {
+	$msg =~ s/\n$/ (Unsupported development snapshot)\n/;
     }
-    return $res if $level eq "short";
 
-    $res->{general}{perl} =
-      { version => "$^V",
-	path    => $^X,
-	dppath  => $cp->display($^X),
-      };
+    if ( $short ) {
+	$msg =~ s/^\s+//;
+	$msg =~ s/\s+/ /g;
+	$msg =~ s/\s*\n//;
+	return $msg;
+    }
 
-    $res->{general}{packager} =
-      { packager => CP->packager,
-	version  => CP->packager_version,
-      } if CP->packager;
+    $msg .= sprintf( $fmtv, "Perl", $^V );
+    $msg =~ s/\n$/ ($^X)\n/;
+    if ( $App::Packager::PACKAGED ) {
+	my $p = App::Packager::Packager();
+	$p .= " Packager" unless $p =~ /packager/i;
+	$msg .= sprintf( $fmtv, $p, $dd->(App::Packager::Version()) );
+    }
 
     # Determine resource path.
-    $res->{general}{library} = [];
-    if ( defined $ENV{CHORDPRO_LIB} ) {
-	for ( $cp->path($ENV{CHORDPRO_LIB}) ) {
-	    push( @{$res->{general}{library}},
-		  { path   => $_,
-		    dppath => $cp->display($_) } );
-	}
-    }
-
-    if ( defined $ENV{FONTDIR} ) {
-	for ( $cp->path($ENV{FONTDIR}) ) {
-	    push( @{$res->{general}{fontdir}},
-		  { path   => $_,
-		    dppath => $cp->display($_) } );
-	}
-    }
-
-    $res->{resources} =
-      [ map { { path => $_, dppath => $cp->display($_) } }
-	    uniq( @{ $cp->resdirs } ) ];
-
-    eval {
-	require ChordPro::Delegate::ABC;
-	my $x = ChordPro::Delegate::ABC->info();
-	$res->{general}{abc} = $x->{info} if $x->{info};
-	1;
-    } or $@ =~ /Can't locate/ or warn($@);
-
     my @p;
+    if ( $ENV{CHORDPRO_LIB} ) {
+	$msg .= sprintf( $fmtvv, "CHORDPRO_LIB", $ENV{CHORDPRO_LIB} );
+	@p = splitpath($ENV{CHORDPRO_LIB});
+    }
+    push( @p, realpath( App::Packager::GetResourcePath() ) );
+    my $tag = "Resource path";
+    for ( @p ) {
+	$msg .= sprintf( $fmtvv, $tag, $_ );
+	$tag = "";
+    }
 
-    my $vv = sub {
-	my ( $mod ) = @_;
-	no strict 'refs';
-	my $pm = $mod =~ s;::;/;gr . ".pm";
-	my $loc = $INC{$pm};
-	push( @p, { name    => $mod,
-		    version => "".$dd->(${${"${mod}::"}{VERSION}}),
-		    path    => $loc,
-		    dppath  => $loc =~ /^(.*)\/\Q$pm\E$/ ? $cp->display($1) : $loc,
-		  } );
-    };
-
+    $msg .= "\nModules and libraries:\n";
     if ( defined $Wx::VERSION ) {
 	no strict 'subs';
-	push( @p,
-	      { name => "wxPerl",    version => $dd->($Wx::VERSION)  },
-	      { name => "wxWidgets", version => $dd->($Wx::wxVERSION) } );
+	$msg .= sprintf( $fmtv, "wxPerl", $dd->($Wx::VERSION) );
+	$msg .= sprintf( $fmtv, "wxWidgets", $dd->(Wx::wxVERSION) );
     }
 
     local $SIG{__WARN__} = sub {};
     local $SIG{__DIE__} = sub {};
-    $vv->("Storable");
-    $vv->("Object::Pad");
-    eval {
-	require Text::Layout;
-	$vv->("Text::Layout");
+    $msg .= sprintf( $fmtv, "Storable", $dd->($Storable::VERSION) );
+    $msg .= sprintf( $fmtv, "Object::Pad", $dd->($Object::Pad::VERSION) );
+    eval { require Text::Layout;
+	$msg .= sprintf( $fmtv, "Text::Layout", $dd->($Text::Layout::VERSION) );
     };
-    eval {
-	require HarfBuzz::Shaper;
-	$vv->("HarfBuzz::Shaper");
-	$p[-1]->{library} = $dd->(HarfBuzz::Shaper::hb_version_string());
+    eval { require HarfBuzz::Shaper;
+	$msg .= sprintf( $fmtv, "HarfBuzz::Shaper", $dd->($HarfBuzz::Shaper::VERSION) );
+	$msg .= sprintf( $fmtv, "HarfBuzz library", $dd->(HarfBuzz::Shaper::hb_version_string()) );
     };
-    eval {
-	require String::Interpolate::Named;
-	$vv->("String::Interpolate::Named");
+    $msg .= sprintf( $fmtv, "File::LoadLines", $dd->($File::LoadLines::VERSION) );
+    eval { require PDF::Builder;
+	$msg .= sprintf( $fmtv, "PDF::Builder", $dd->($PDF::Builder::VERSION) );
+    }
+    or
+    eval { require PDF::API2;
+	$msg .= sprintf( $fmtv, "PDF::API2", $dd->($PDF::API2::VERSION) );
     };
-    eval {
-	require File::LoadLines;
-	$vv->("File::LoadLines");
+    eval { require Font::TTF;
+	$msg .= sprintf( $fmtv, "Font::TTF", $dd->($Font::TTF::VERSION) );
     };
-    eval {
-	require PDF::Builder;
-	$vv->("PDF::Builder");
+    eval { require Image::Magick;
+	$msg .= sprintf( $fmtv, "Image::Magick",
+			 $dd->( $Image::Magick::VERSION ||
+				$Image::Magick::Q16::VERSION ||
+				$Image::Magick::Q8::VERSION || "6.x?" ) );
     };
-    eval {
-	require PDF::API2;
-	$vv->("PDF::API2");
-	eval {
-	    require PDF::API2::XS;
-	    $vv->("PDF::API2::XS");
-	};
-    };
-    eval {
-	require SVGPDF;
-	$vv->("SVGPDF");
-    };
-    eval {
-	require Font::TTF;
-	$vv->("Font::TTF");
-    };
-    eval {
-	require JavaScript::QuickJS;
-	$vv->("JavaScript::QuickJS");
-    };
-    my $i = json_parser();
-    $vv->( $i->{parser} );
-    $p[-1]->{relaxed} = "relaxed" if $i->{relaxed};
-
-    eval {
-	require JSON::XS;
-	$vv->("JSON::XS");
-    };
-
-    $res->{modules} = [ @p ];
-
-    $res->{metadata} = $::config->{metadata}{keys};
-
-    @p = ( qw(title subtitle),
-	   ( grep { !/^(sub)?title$/ } sort(@{$::config->{metadata}{keys}//[]}) ),
-	   grep { !/^(sub)?title$/ } (keys(%{ChordPro::Song::_directives()})) );
-    $res->{directives} = [ @p ];
-
-    $res->{directive_abbrevs} = ChordPro::Song::_directive_abbrevs();
-
-    return $res;
+    return $msg;
 }
 
 sub splitpath {
@@ -1398,25 +1145,6 @@ sub app_usage {
     for ( qw( config userconfig sysconfig) ) {
 	$cfg{$_} = $configs{$_} || "no default";
     }
-
-    if ( $::options->{reference} ) {
-	print ${fh} <<EndOfUsage;
-Usage: $0 [ options ] [ file ... ]
-
-Options:
-    --[no]strict        Strict conformance
-    --output=FILE  -o   Saves the output to FILE
-    --about  -A         About ChordPro...
-    --version  -V       Prints version and exits
-    --help  -h          This message
-    --ident             Show identification
-    --verbose           Verbose information. Repeat for more.
-EndOfUsage
-	exit $exit if defined $exit;
-	return;
-    }
-
-    ####TODO: weed out for --reference.
     print ${fh} <<EndOfUsage;
 Usage: $0 [ options ] [ file ... ]
 
@@ -1434,32 +1162,31 @@ Options:
     --encoding=ENC                Encoding for input files (UTF-8)
     --filelist=FILE               Reads song file names from FILE
     --fragment -F                 Partial (incomplete) song
-    --front-matter=FILE           Add front matter pages from PDF document
+    --front-matter=FILE           Add cover pages from PDF document
     --lyrics-only  -l             Only prints lyrics
     --meta KEY=VALUE              Add meta data
     --output=FILE  -o             Saves the output to FILE
     --[no]strict                  Strict conformance
     --start-page-number=N  -p     Starting page number [1]
-    --subtitle=VALUE		  Use with --title for a nice subtitle.
-    --title=VALUE		  Adds a nice cover page using this title.
     --toc --notoc -i              Generates/suppresses a table of contents
     --transcode=SYS  -xc          Transcodes to notation system
     --transpose=N  -x             Transposes by N semi-tones
     --version  -V                 Prints version and exits
 
 Chordii compatibility.
-Options marked with * are better handled in the config file.
+Options marked with * are better specified in the config file.
+Options marked with - are ignored.
     --chord-font=FONT  -C         *Sets chord font
     --chord-grid-size=N  -s       *Sets chord diagram size [30]
     --chord-grids-sorted  -S      *Prints chord diagrams ordered by key
     --chord-size=N  -c            *Sets chord size [9]
-    --dump-chords  -D             Dumps chords definitions (PDF)
+    --dump-chords  -D             Dumps chords definitions (PostScript)
     --dump-chords-text  -d        Dumps chords definitions (Text)
-    --even-pages-number-left  -L  *Not supported
-    --odd-pages-number-left       *Not supported.
+    --even-pages-number-left  -L  *Even pages numbers on left
+    --odd-pages-number-left       *Odd pages numbers on left
     --no-chord-grids  -G          *Disables printing of chord diagrams
     --no-easy-chord-grids  -g     Not supported
-    --page-number-logical  -n     Not supported
+    --page-number-logical  -n     -Numbers logical pages, not physical
     --page-size=FMT  -P           *Specifies page size [letter, a4 (default)]
     --single-space  -a            *Automatic single space lines without chords
     --text-size=N  -t             *Sets text size [12]
@@ -1495,6 +1222,40 @@ EndOfUsage
     exit $exit if defined $exit;
 }
 
+################ Resources ################
+
+use Encode qw(decode decode_utf8 encode_utf8);
+
+sub ::rsc_or_file {
+    my ( $c, $cfg ) = @_;
+    my $f = $c;
+    $cfg .= "/" if $cfg;
+
+    # Check for resource names.
+    if ( $f !~ m;[/.]; ) {
+	if ( $c =~ /^(.+):(.*)/ ) {
+	    $f = $cfg . lc($1) . "/" . lc($2) . ".json";
+	}
+	else {
+	    $f = $cfg . lc($c) . ".json";
+	}
+    }
+    if ( $ENV{CHORDPRO_LIB} ) {
+	my @libs = splitpath($ENV{CHORDPRO_LIB});
+	foreach my $lib ( @libs ) {
+	    $lib = expand_tilde($lib);
+	    warn("RSC1: $lib/$f\n") if $options->{debug};
+	    return $lib . "/" . $f if -r $lib . "/" . $f;
+	}
+    }
+
+    warn("RSC3: $f\n") if $options->{debug};
+    my $t = getresource($f);
+    return defined($t) ? $t : $c;
+}
+
+use lib ( grep { defined } getresource("CPAN") );
+
 =head1 FONTS
 
 There are two ways to specify fonts: with a font filename, and a
@@ -1507,7 +1268,9 @@ dir, and the contents of environment variable C<FONTDIR>. In any case,
 the filename should point to a valid TrueType (C<.ttf>) or OpenType
 (C<.otf>) font.
 
-If it is not a filename, it must be the name one of the built-in PDF core fonts:
+If it is not a filename, it must be the name one of the built-in fonts.
+
+Built-in 'Adobe Core Fonts':
 
   Courier                             Symbol
   Courier-Bold                        Times-Bold
@@ -1518,6 +1281,17 @@ If it is not a filename, it must be the name one of the built-in PDF core fonts:
   Helvetica-BoldOblique
   Helvetica-Oblique
 
+Built-in 'Windows Fonts':
+
+  Georgia                             Webdings
+  Georgia,Bold                        Wingdings
+  Georgia,BoldItalic
+  Georgia,Italic
+  Verdana
+  Verdana,Bold
+  Verdana,BoldItalic
+  Verdana,Italic
+
 =head1 MOTIVATION
 
 Why a rewrite of Chordii?
