@@ -142,7 +142,7 @@ sub get_parser ( $self, $system = undef, $nofallback = undef ) {
     }
     elsif ( $system ne $::config->{notes}->{system} ) {
 	my $p = ChordPro::Chords::Parser::Common->new
-	  ( { notes => { system => $system } } );
+	  ( { notes => $system } );
 	return $parsers{$system} = $p;
     }
     elsif ( $system ) {
@@ -248,7 +248,7 @@ sub parse_chord ( $self, $chord ) {
 
     my $q = $plus{qual} // "";
     $info->{qual} = $q;
-    $q = "-" if $q eq "m" || $q eq "mi" || $q eq "min";
+    $q = "-" if $q eq "m" || $q eq "min";
     $q = "+" if $q eq "aug";
     $q = "0" if $q eq "dim";
     $q = "0" if $q eq "o";
@@ -410,7 +410,6 @@ my $additions_min =
    "69",
    "7b5",
    "7-5",
-   "711",			# for George Kooymans
    ( map { ( "$_", "maj$_", "^$_" ) }
      "7",
      "9",
@@ -493,7 +492,7 @@ sub load_notes ( $self, $init ) {
     # Accept root, qual, and only known extensions.
     $c_pat = "(?<root>" . $n_pat . ")";
     $c_pat .= "(?:";
-    $c_pat .= "(?<qual>-|min?|m(?!aj))".
+    $c_pat .= "(?<qual>-|min|m(?!aj))".
       "(?<ext>" . join("|", keys(%$additions_min)) . ")|";
     $c_pat .= "(?<qual>\\+|aug)".
       "(?<ext>" . join("|", keys(%$additions_aug)) . ")|";
@@ -507,7 +506,7 @@ sub load_notes ( $self, $init ) {
 
     # In relaxed form, we accept anything for extension.
     my $c_rpat = "(?<root>" . $n_pat . ")";
-    $c_rpat .= "(?:(?<qual>-|min?|m(?!aj)|\\+|aug|0|o|dim|)(?<ext>.*))";
+    $c_rpat .= "(?:(?<qual>-|min|m(?!aj)|\\+|aug|0|o|dim|)(?<ext>.*))";
     $c_rpat = qr/$c_rpat/;
 
     # Store in the object.
@@ -544,7 +543,6 @@ sub new {
     my $self = bless { chord_cache => {} } => $pkg;
     $self->{system} = "nashville";
     $self->{target} = 'ChordPro::Chord::Nashville';
-    $self->{intervals} = 12;
     warn("Chords: Created parser for ", $self->{system}, "\n")
       if $::options->{verbose} && $::options->{verbose} > 1;
     return $parsers{$self->{system}} = $self;
@@ -657,7 +655,6 @@ sub new {
     my $self = bless { chord_cache => {} } => $pkg;
     $self->{system} = "roman";
     $self->{target} = 'ChordPro::Chord::Roman';
-    $self->{intervals} = 12;
     warn("Chords: Created parser for ", $self->{system}, "\n")
       if $::options->{verbose} && $::options->{verbose} > 1;
     return $parsers{$self->{system}} = $self;
@@ -778,11 +775,12 @@ sub clone {
 
 sub is_note { $_[0]->{isnote} };
 sub is_flat { $_[0]->{isflat} };
+sub is_keyboard { $_[0]->{iskeyboard} };
 
 sub is_nc {
     my ( $self ) = @_;
     # Keyboard...
-    return 1 if defined($self->kbkeys) && !@{$self->kbkeys};
+    return 1 if $self->is_keyboard && !@{ $self->kbkeys // [1] };
     # Strings...
     return unless @{ $self->frets // [] };
     for ( @{ $self->frets } ) {
@@ -808,7 +806,6 @@ sub is_chord      { defined $_[0]->{root_ord} }
 sub is_rootless   { $_[0]->{rootless} }
 sub is_annotation { 0 }
 sub is_movable    { $_[0]->{movable} }
-sub is_gridstrum  { 0 }
 
 # Common accessors.
 sub name          {
@@ -841,41 +838,10 @@ sub kbkeys {
     $_[0]->{keys} = ChordPro::Chords::get_keys($_[0]);
 }
 
-sub chord_display ( $self, $default ) {
-
-    use String::Interpolate::Named;
-
-    my $res = $self->name;
-    my $args = {};
-    $self->flat_copy( $args, $self->{display} // $self );
-
-    if ( !$::config->{settings}->{'enharmonic-transpose'} && $args->{key} ) {
-	$args->{root} = 'E#'
-	  if $args->{root} eq 'F' && $args->{key} eq 'F#';
-	$args->{root} = 'Cb'
-	  if $args->{root} eq 'B' && $args->{key} eq 'Gb';
-    }
-
-    for my $fmt ( $default,
-		  $self->{format},
-		  $self->{chordformat} ) {
-	next unless $fmt;
-	$args->{root} = lc($args->{root}) if $self->is_note;
-	$args->{formatted} = $res;
-	$res = interpolate( { args => $args }, $fmt );
-    }
-
-    # Substitute musical symbols if wanted.
-    $res = $self->fix_musicsyms($res)
-      if $::config->{settings}->{truesf} || $::config->{settings}->{maj7delta};
-    return $res;
-}
-
 sub flat_copy ( $self, $ret, $o, $pfx = "" ) {
     while ( my ( $k, $v ) = each %$o ) {
-	if ( $k eq "orig" || $k eq "xc" || $k eq "xp" ) {
+	if ( $k eq "orig" ) {
 	    $self->flat_copy( $ret, $v, "$k.$pfx");
-	    $ret->{"$k.${pfx}formatted"} = $v->chord_display;
 	}
 	else {
 	    $ret->{"$pfx$k"} = $v;
@@ -888,30 +854,23 @@ sub fix_musicsyms ( $self, $str ) {
 
     use ChordPro::Utils qw( splitmarkup );
 
-    my $sf = $::config->{settings}->{truesf};
-    my $delta = $::config->{settings}->{maj7delta};
+    return $str unless $::config->{settings}->{truesf};
 
     my @c = splitmarkup($str);
     my $res = '';
     push( @c, '' ) if @c % 2;
     my $did = 0;		# TODO: not for roman
     while ( @c ) {
-	for ( shift(@c) ) {
-	    if ( $sf ) {
-		if ( $did ) {
-		    s/b/♭/g;
-		}
-		else {
-		    s/(?<=[[:alnum:]])b/♭/g;
-		    $did++;
-		}
-		s/#/♯/g;
-	    }
-	    if ( $delta ) {
-		s/maj7/Δ/g;
-	    }
-	    $res .= $_ . shift(@c);
+	$_ = shift(@c);
+	if ( $did ) {
+	    s/b/♭/g;
 	}
+	else {
+	    s/(?<=[[:alnum:]])b/♭/g;
+	    $did++;
+	}
+	s/#/♯/g;
+	$res .= $_ . shift(@c);
     }
     $res;
 }
@@ -942,6 +901,8 @@ package ChordPro::Chord::Common;
 
 our @ISA = qw( ChordPro::Chord::Base );
 
+use String::Interpolate::Named;
+
 # Show reconstructs the chord from its constituents.
 # Result is canonical.
 sub show {
@@ -991,11 +952,6 @@ sub transpose ( $self, $xpose, $dir = 0 ) {
     my $info = $self->clone;
     my $p = $self->{parser};
 
-    my $dodir = sub( $root, $dir ) {
-	return 0 if $root =~ /^(0|2|4|5|7|9|11)$/;
-	$dir;
-    };
-
     unless ( $self->{rootless} ) {
 	$info->{root_ord} = ( $self->{root_ord} + $xpose ) % $p->intervals;
 	$info->{root_canon} = $info->{root} =
@@ -1007,9 +963,9 @@ sub transpose ( $self, $xpose, $dir = 0 ) {
 	$info->{bass_ord} = ( $self->{bass_ord} + $xpose ) % $p->intervals;
 	$info->{bass_canon} = $info->{bass} =
 	  $p->root_canon( $info->{bass_ord}, $xpose > 0 );
-	$info->{bass_mod} = $dodir->( $info->{bass_ord}, $dir );
+	$info->{bass_mod} = $dir;
     }
-    $info->{root_mod} = $dodir->( $info->{root_ord}, $dir );
+    $info->{root_mod} = $dir;
     $info->{name} = $info->{name_canon} = $info->canonical;
 
     delete $info->{$_} for qw( copy base frets fingers keys display );
@@ -1027,24 +983,18 @@ sub transcode ( $self, $xcode, $key_ord = 0 ) {
     my $p = $self->{parser}->get_parser($xcode);
     die("OOPS ", $p->{system}, " $xcode") unless $p->{system} eq $xcode;
     $info->{parser} = $p;
-    if ( $key_ord && $p->movable ) {
-	$info->{root_ord} -= $key_ord % $p->intervals;
-    }
+    $info->{root_ord} -= $key_ord if $key_ord && $p->movable;
 #    $info->{$_} = $p->{$_} for qw( ns_tbl nf_tbl ns_canon nf_canon );
-    unless ( $self->{rootless} ) {
-	$info->{root_canon} = $info->{root} =
-	  $p->root_canon( $info->{root_ord},
-			  $info->{root_mod} >= 0,
-			  $info->{qual_canon} eq "-" );
-    }
+    $info->{root_canon} = $info->{root} =
+      $p->root_canon( $info->{root_ord},
+		      $info->{root_mod} >= 0,
+		      $info->{qual_canon} eq "-" );
     if ( $p->{system} eq "roman" && $info->{qual_canon} eq "-" ) {
 	# Minor quality is in the root name.
 	$info->{qual_canon} = $info->{qual} = "";
     }
     if ( $self->{bass} && $self->{bass} ne "" ) {
-	if ( $key_ord && $p->movable ) {
-	    $info->{bass_ord} -= $key_ord % $p->intervals;
-	}
+	$info->{bass_ord} -= $key_ord if $key_ord && $p->movable;
 	$info->{bass_canon} = $info->{bass} =
 	  $p->root_canon( $info->{bass_ord}, $info->{bass_mod} >= 0 );
     }
@@ -1058,10 +1008,23 @@ sub transcode ( $self, $xcode, $key_ord = 0 ) {
 
 sub chord_display ( $self ) {
 
-    $self->SUPER::chord_display
-      ( $::config->{"chord-formats"}->{common}
-	// $::config->{settings}->{"chord-format"}
-	// "%{name}" );
+    # $self->dump;
+
+    my $res = $self->name;
+    my $args = {};
+    $self->flat_copy( $args, $self->{display} // $self );
+
+    for my $fmt ( $::config->{settings}->{"chord-format"},
+		  $self->{format},
+		  $self->{chordformat} ) {
+	next unless $fmt;
+	$args->{root} = lc($args->{root}) if $self->is_note;
+	$args->{formatted} = $res;
+	$res = interpolate( { args => $args }, $fmt );
+    }
+
+    # Substitute musical symbols if wanted.
+    return $::config->{settings}->{truesf} ? $self->fix_musicsyms($res) : $res;
 }
 
 ################ Chord objects: Nashville ################
@@ -1070,7 +1033,9 @@ package ChordPro::Chord::Nashville;
 
 our @ISA = 'ChordPro::Chord::Base';
 
-sub transpose ( $self, $dummy1, $dummy2=0 ) { $self }
+use String::Interpolate::Named;
+
+sub transpose ( $self ) { $self }
 
 sub show {
     Carp::croak("call canonical instead of show");
@@ -1085,10 +1050,22 @@ sub canonical ( $self ) {
 }
 
 sub chord_display ( $self ) {
+    if ( $self->{format} ) {
+	####TODO
+	my $fmt = $self->{format} || $::config->{settings}->{"chord-format"};
+	if ( $fmt ) {
+	    my $args = {};
+	    $self->flat_copy( $args, $self );
+	    return interpolate( { args => $args }, $fmt );
+	}
+    }
 
-    $self->SUPER::chord_display
-      ( $::config->{"chord-formats"}->{nashville}
-	// "%{name}" );
+    my $res = $self->{root_canon} .
+      "<sup>" . $self->{qual} . $self->{ext} . "</sup>";
+    if ( $self->{bass} && $self->{bass} ne "" ) {
+	$res .= "<sub>/" . lc($self->{bass}) . "</sub>";
+    }
+    return $res;
 }
 
 ################ Chord objects: Roman ################
@@ -1097,7 +1074,9 @@ package ChordPro::Chord::Roman;
 
 our @ISA = 'ChordPro::Chord::Base';
 
-sub transpose ( $self, $dummy1, $dummy2=0 ) { $self }
+use String::Interpolate::Named;
+
+sub transpose ( $self ) { $self }
 
 sub show {
     Carp::croak("call canonical instead of show");
@@ -1112,10 +1091,24 @@ sub canonical ( $self ) {
 }
 
 sub chord_display ( $self ) {
+    if ( $self->{format} ) {
+	####TODO
+	my $fmt = $self->{format} || $::config->{settings}->{"chord-format"};
+	if ( $fmt ) {
+	    my $args = {};
+	    $self->flat_copy( $args, $self );
+	    return interpolate( { args => $args }, $fmt );
+	}
+	return $self->canonical;
+    }
 
-    $self->SUPER::chord_display
-      ( $::config->{"chord-formats"}->{roman}
-	// "%{name}" );
+    my $res = $self->{root_canon};
+    $res .= "<sup>" . $self->{qual} . $self->{ext} . "</sup>"
+      if $self->{qual};
+    if ( $self->{bass} && $self->{bass} ne "" ) {
+	$res .= "<sub>/" . lc($self->{bass}) . "</sub>";
+    }
+    return $res;
 }
 
 ################ Chord objects: Annotations ################
@@ -1126,8 +1119,8 @@ use String::Interpolate::Named;
 
 our @ISA = 'ChordPro::Chord::Base';
 
-sub transpose ( $self, $dummy1, $dummy2=0 ) { $self }
-sub transcode ( $self, $dummy1, $dummy2=0 ) { $self }
+sub transpose ( $self ) { $self }
+sub transcode ( $self ) { $self }
 
 sub canonical ( $self ) {
     my $res = $self->{text};
@@ -1142,77 +1135,6 @@ sub chord_display ( $self ) {
 sub is_chord      ( $self ) { 0 };
 sub is_annotation ( $self ) { 1 };
 
-################ Chord objects: Strums ################
-
-package ChordPro::Chord::Strum;
-
-# Special 'chord'-like objects for strums in grids.
-#
-# Main purpose is to show an arrow from the ChordProSymbols font.
-
-our @ISA = 'ChordPro::Chord::Base';
-
-use ChordPro::Symbols qw( strum );
-
-sub new( $pkg, $data ) {
-    my $self = $pkg->SUPER::new( $data );
-    my $fmt = strum( $data->{name} );
-    unless ( defined $fmt ) {
-	warn("Unknown strum: $data->{name}\n");
-	$self->{format} = "";
-    }
-    else {
-	$self->{format} = $fmt;
-    }
-    return $self;
-}
-
-sub chord_display ( $self, $default = undef ) {
-    $self->{format};
-}
-
-sub transpose ( $self, $dummy1, $dummy2=0 ) { $self }
-sub transcode ( $self, $dummy1, $dummy2=0 ) { $self }
-
-sub canonical ( $self ) {
-    my $res = $self->{text};
-    return $res;
-}
-
-# For convenience.
-sub is_chord      ( $self ) { 0 };
-sub is_annotation ( $self ) { 1 };
-sub is_nc         ( $self ) { 1 };
-sub is_xpxc       ( $self ) { 0 };
-sub has_diagram   ( $self ) { 0 };
-sub is_gridstrum  ( $self ) { 1 };
-
-################ Chord objects: NC ################
-
-package ChordPro::Chord::NC;
-
-use String::Interpolate::Named;
-
-our @ISA = 'ChordPro::Chord::Base';
-
-sub transpose ( $self, $dummy1, $dummy2=0 ) { $self }
-sub transcode ( $self, $dummy1, $dummy2=0 ) { $self }
-
-sub canonical ( $self ) {
-    my $res = $self->{name};
-    return $res;
-}
-
-sub chord_display ( $self ) {
-    return interpolate( { args => $self }, $self->{name} );
-}
-
-# For convenience.
-sub is_nc         ( $self ) { 1 };
-sub is_chord      ( $self ) { 0 };
-sub is_annotation ( $self ) { 0 };
-sub has_diagram   ( $self ) { 0 };
-
 ################ Testing ################
 
 package main;
