@@ -9,21 +9,14 @@ package ChordPro::Songbook;
 
 use strict;
 use warnings;
-use feature 'state';
 
 use ChordPro;
 use ChordPro::Config;
-use ChordPro::Files;
 use ChordPro::Song;
-use ChordPro::Utils qw(progress);
 
 use Carp;
 use List::Util qw(any);
-use Storable qw(dclone);
-use Ref::Util qw(is_arrayref is_plain_hashref);
-use MIME::Base64;
-
-my $regtest = defined($ENV{PERL_HASH_SEED}) && $ENV{PERL_HASH_SEED} == 0;
+use File::LoadLines;
 
 sub new {
     my ($pkg) = @_;
@@ -41,12 +34,8 @@ sub parse_file {
 	return $self->embed_file( $filename, $meta, $defs );
     }
 
-    # fs_load sets $opts->{_filesource}.
-    $opts->{fail} = "soft";
-    my $lines = is_arrayref($filename) ? $filename
-      : fs_load( $filename, $opts );
-    die( $filename, ": ", $opts->{error}, "\n" ) if $opts->{error};
-
+    # Loadlines sets $opts->{_filesource}.
+    my $lines = loadlines( $filename, $opts );
     # Sense crd input and convert if necessary.
     if ( !(defined($options->{a2crd}) && !$options->{a2crd}) and
 	 !$options->{fragment}
@@ -69,40 +58,17 @@ sub parse_file {
 	next unless exists $opts->{$_};
 	$options->{$_} = $opts->{$_};
     }
-    bless $config => ChordPro::Config:: if is_plain_hashref($config);
+    bless $config => ChordPro::Config:: if ref $config eq 'HASH';
 
     my $linecnt = 0;
     my $songs = 0;
-
     while ( @$lines ) {
-	my $song = ChordPro::Song->new($opts)
-	  ->parse_song( $lines, \$linecnt,
-			{ %{dclone($meta)},
-			  "bookmark"   => $opts->{bookmark} //= sprintf( "song_%d", 1 + @{ $self->{songs} } ),
-			},
-			{ %$defs } );
-
+	my $song = ChordPro::Song
+	  ->new( $opts->{_filesource} )
+	  ->parse_song( $lines, \$linecnt, {%$meta}, {%$defs} );
 	$song->{meta}->{songindex} = 1 + @{ $self->{songs} };
 	push( @{ $self->{songs} }, $song );
-	$songs++;
-
-	# Copy persistent assets to the songbook.
-	if ( $song->{assets} ) {
-	    $self->{assets} //= {};
-	    while ( my ($k,$v) = each %{$song->{assets}} ) {
-		next unless $v->{opts} && $v->{opts}->{persist};
-		$self->{assets}->{$k} = $v;
-	    }
-	}
-    }
-
-    if ( @{$self->{songs}} > 1 ) {
-	my $song = $self->{songs}->[-1];
-	unless ( $song->{body}
-		 && any { $_->{type} ne "ignore" } @{$song->{body}} ) {
-	    pop( @{ $self->{songs} } );
-	    $songs--;
-	}
+	$songs++ if $song->{body};
     }
 
     warn("Warning: No songs found in ", $opts->{_filesource}, "\n")
@@ -111,22 +77,16 @@ sub parse_file {
     return 1;
 }
 
-sub add {
-    my ( $self, $song ) = @_;
-    push( @{$self->{songs}}, $song );
-    $self;
-}
-
 sub embed_file {
     my ( $self, $filename, $meta, $defs ) = @_;
 
-    unless ( fs_test( sr => $filename ) ) {
+    unless ( -s -r $filename ) {
 	warn("$filename: $! (skipped)\n");
 	return;
     }
     my $type = "pdf";
 
-    my $song = ChordPro::Song->new( { filesource => $filename } );
+    my $song = ChordPro::Song->new( $filename );
     $song->{meta}->{songindex} = 1 + @{ $self->{songs} };
     $song->{source} =
       { file      => $filename,
